@@ -1,5 +1,5 @@
-# License: Public Domain or CC0
-# See https://creativecommons.org/publicdomain/zero/1.0/
+# License: Public Domain or CC0 See
+# https://creativecommons.org/publicdomain/zero/1.0/
 # The author, Jim Avera (jim.avera at gmail) has waived all copyright and
 # related or neighboring rights.  Attribution is requested but is not required.
 
@@ -12,6 +12,7 @@ use strict; use warnings FATAL => 'all'; use utf8;
 use 5.018;  # lexical_subs
 use feature qw(say state lexical_subs current_sub);
 use feature 'lexical_subs';
+use feature 'unicode_strings';
 
 no warnings "experimental::lexical_subs";
 
@@ -26,8 +27,8 @@ package
 package Data::Dumper::Interp;
 
 { no strict 'refs'; ${__PACKAGE__."::VER"."SION"} = 997.999; }
-our $VERSION = '7.001'; # VERSION from Dist::Zilla::Plugin::OurPkgVersion
-our $DATE = '2023-12-13'; # DATE from Dist::Zilla::Plugin::OurDate
+our $VERSION = '7.002'; # VERSION from Dist::Zilla::Plugin::OurPkgVersion
+our $DATE = '2024-01-08'; # DATE from Dist::Zilla::Plugin::OurDate
 
 use Moose;
 
@@ -75,6 +76,8 @@ our %EXPORT_TAGS = (
 
 sub _generate_sub($;$); # forward
 
+our ($COND_LB, $COND_RB, $COND_MULT, $LQ, $RQ);
+
 #---------------------------------------------------------------------------
 my $sane_cW = $^W;
 my $sane_cH = $^H;
@@ -215,7 +218,7 @@ sub _dbshow(_) {
               : _dbvis($v)               # something else
 }
 our $_dbmaxlen = 300;
-sub _dbrawstr(_) { "«".(length($_[0])>$_dbmaxlen ? substr($_[0],0,$_dbmaxlen-3)."..." : $_[0])."»" }
+sub _dbrawstr(_) { "${LQ}".(length($_[0])>$_dbmaxlen ? substr($_[0],0,$_dbmaxlen-3)."..." : $_[0])."${RQ}" }
 sub _dbstr($) {
   local $_ = shift;
   return "undef" if !defined;
@@ -482,13 +485,14 @@ sub __getself_h {
 
 sub _EnabUseqqFeature {
   # Append <feature> to Useqq ONLY if Useqq has not been changed
-  # (indicated by "(parens)" in the value -- see $Useqq = ... )
+  # (indicated by "<pointy brackets>" in the value -- see $Useqq = ... )
   # Does nothing unless the default enables extended features.
   my ($self, $feature) = @_;
   my $curr = $self->Useqq;
   return $self if length($curr//"") <= 1
                     || substr($curr,0,1) ne "<"
                     || substr($curr,-1,1) ne ">";
+#btw '###ENABLE CHANGING Useqq; curr=', _dbvis($curr), "  \$Useqq=$Useqq";
   $self->Useqq($curr.$feature)
 }
 
@@ -507,21 +511,22 @@ sub _generate_sub($;$) {
   s/alvis/avisl/;  # backwards compat.
   s/hlvis/hvisl/;  # backwards compat.
 
-  s/^[^diha]*\K(?:lvis|visl)/avisl/; # 'visl' same as 'avisl' for bw compat.
+  # NOW visl means something else.
+  #s/^[^diha]*\K(?:lvis|visl)/avisl/; # 'visl' same as 'avisl' for bw compat.
 
   s/([ahid]?vis)// or error "can not infer the basic function";
   my $basename = $1;  # avis, hvis, ivis, dvis, or vis
   my $N = s/(\d+)// ? $1 : undef;
   my %mod = map{$_ => 1} split //, $_;
-  delete $mod{"_"}; # ignore underscores
+  delete $mod{"_"}; # ignore underscores in names
 
   if (($Debug//0) > 1) {
     warn "## (D=$Debug) methname=$methname base=$basename \$_=$_\n";
   }
-  if ($basename =~ /^[id]/) {
-    error "'$1' is inapplicable to $basename" if /([ahl])/;
-  }
-  error "'$1' mis-placed: Only allowed as '${1}vis'" if /([ahi])/;
+##  if ($basename =~ /^[id]/) {
+##    error "'$1' is inapplicable to $basename" if /([ahl])/;
+##  }
+##  error "'$1' mis-placed: Only allowed as '${1}vis'" if /([ahi])/;
 
 
   # All these subs can be called as either or methods or functions.
@@ -541,15 +546,16 @@ sub _generate_sub($;$) {
     $code .= ";";
   } else {
     if ($basename eq "vis") {
-      $code .= " { &__getself_s->_Listform('')";
+      my $listform = delete($mod{l}) ? 'l' : '';
+      $code .= " { &__getself_s->_Listform('${listform}')";
     }
     elsif ($basename eq "avis") {
       my $listform = delete($mod{l}) ? 'l' : 'a';
-      $code .= " { &__getself_a->_Listform('$listform')";
+      $code .= " { &__getself_a->_Listform('${listform}')";
     }
     elsif ($basename eq "hvis") {
       my $listform = delete($mod{l}) ? 'l' : 'h';
-      $code .= " { &__getself_h->_Listform('$listform')";
+      $code .= " { &__getself_h->_Listform('${listform}')";
     }
     elsif ($basename eq "ivis") {
       $code .= " { \@_ = ( &__getself" ;
@@ -560,12 +566,20 @@ sub _generate_sub($;$) {
     }
     else { oops }
 
+    my $useqq = "";
+    $useqq .= ":unicode:controlpics" if delete $mod{c};
+    $useqq .= ":condense"            if delete $mod{C};
+    $code .= '->Debug(2)'            if delete $mod{d};
+    $useqq .= ":hex"                 if delete $mod{h};
+    $code .= '->Objects(0)'          if delete $mod{o};
+    $useqq .= ":octets"              if delete $mod{O};
+    $code .= '->Refaddr(1)'          if delete $mod{r};
+    $useqq .= ":underscores"         if delete $mod{u};
+
+    $code .= "->Useqq(\$Useqq.'${useqq}')" if $useqq ne "";
+    $code .= "->Useqq(0)"     if delete $mod{q};
+
     $code .= "->Maxdepth($N)" if defined($N);
-    $code .= '->Objects(0)'   if delete $mod{o};
-    $code .= '->Useqq(0)'     if delete $mod{q};
-    $code .= '->Useqq("unicode:controlpics")' if delete $mod{c};
-    $code .= '->Refaddr(1)'   if delete $mod{r};
-    $code .= '->Debug(2)'     if delete $mod{d};
 
     if ($basename =~ /^([id])vis/) {
       $code .= ", shift, '$1' ); goto &_Interpolate }";
@@ -670,7 +684,7 @@ sub _Do {
   $truncsuffix //= "...";
   if (ref($objects) eq "HASH") {
     for (qw/show_classname objects/) {
-      croak "Objects value is a hashref but '${_}' key is missing\n"
+      confess "Objects value is a hashref but '${_}' key is missing\n"
         unless exists $objects->{$_};
     }
     $show_classname = $objects->{show_classname};
@@ -1262,9 +1276,9 @@ sub __unesc_unicode() {  # edits $_
     # so is considered "Graphical", but we want to see it as hex rather
     # than "", and probably any other "Format" category Unicode characters.
 
-    s/
+    s{
        \G (?: [^\\]++ | \\[^x] )*+ \K (?<w> \\x\x{7B} (?<hex>[a-fA-F0-9]+) \x{7D} )
-     /
+     }{
        my $orig = $+{w};
        local $_ = hex( length($+{hex}) > 6 ? '0' : $+{hex} );
        $_ = $_ > 0x10FFFF ? "\0" : chr($_); # 10FFFF is Unicode limit
@@ -1272,7 +1286,30 @@ sub __unesc_unicode() {  # edits $_
        # choice of case when escaping wide characters.
        (m<\P{XPosixGraph}|[\0-\177]>
           || m<\p{General_Category=Format}>) ? lc($orig) : $_
-     /xesg;
+     }xesg;
+  }
+}
+
+my %ctlesc2codepoint = (
+  '\\a' => ord("\a"),
+  '\\b' => ord("\b"),
+  '\\t' => ord("\t"),
+  '\\n' => ord("\n"),
+  '\\f' => ord("\f"),
+  '\\r' => ord("\r"),
+  '\\e' => ord("\e"),
+);
+sub __unesc_nonoctal () {  # edits $_
+  # Change backslash escapes like \n back to octal escapes.
+  # This is to better visualize binary octet streams
+  if (/^"/) {
+    s{
+       \G (?: [^\\]++ | \\[x0-7] )*+ \K (?<w> \\[abtnfre])(?<digitnext>\d?)
+     }{
+      $+{digitnext}
+        ? sprintf("\\%03o", ($ctlesc2codepoint{$+{w}} // oops))
+        : sprintf("\\%01o", ($ctlesc2codepoint{$+{w}} // oops))
+     }xesg;
   }
 }
 
@@ -1321,15 +1358,10 @@ sub __subst_spacedots() {  # edits $_
   }
 }
 
-our $COND_LB = "\N{LEFT DOUBLE PARENTHESIS}";
-our $COND_RB = "\N{RIGHT DOUBLE PARENTHESIS}";
-our $COND_MULT = "\N{MULTIPLICATION SIGN}";
-#my $singlechar_restr = "[^\\\\${COND_LB}${COND_RB}]";
-my $singlechar_restr = "[^\\\\${COND_LB}${COND_RB}${COND_MULT}]";
-
 sub __condense_strings($) {  # edits $_
   if (/^"/) {
     my $minrep_m1 = $_[0] - 1;
+    my $singlechar_restr = "[^\\\\${COND_LB}${COND_RB}${COND_MULT}]";
 
     # Special case a string of nul represented as \n\n\n...\00n (n=0..7)
     # D::D generates this to avoid ambiguity if a digit follows
@@ -1348,6 +1380,11 @@ sub __condense_strings($) {  # edits $_
   }
 }
 
+sub __nums_in_hex() {
+  if (looks_like_number($_)) {
+    s/^([1-9]\d+)$/ sprintf("%#x", $1) /e; # Leave single-digit numbers as-is
+  }
+}
 sub __nums_with_underscores() {
   if (looks_like_number($_)) {
     while( s/^([^\._]*?\d)(\d\d\d)(?=$|\.|_)/$1_$2/ ) { }
@@ -1375,9 +1412,11 @@ sub _postprocess_DD_result {
   my $useqq = $self->Useqq();
   my $unesc_unicode = $useqq =~ /utf|unic/;
   my $condense_strings = $useqq =~ /cond/;
+  my $octet_strings = $useqq =~ /octet/;
+  my $nums_in_hex   = $useqq =~ /hex/;
   my $controlpics   = $useqq =~ /pic/;
   my $spacedots     = $useqq =~ /space/;
-  my $underscores   = $useqq =~ /_/;
+  my $underscores   = $useqq =~ /under/;
   my $qq            = $useqq =~ /qq(?:=(..))?/ ? ($1//'{}') : '';
   my $pad = $self->Pad() // "";
 
@@ -1386,6 +1425,22 @@ sub _postprocess_DD_result {
   my $maxlinelen = $foldwidth1 || $foldwidth || INT_MAX;
   my $maxlineNlen = ($foldwidth // INT_MAX) - length($pad);
 
+  state $utf_output = grep /utf/i, PerlIO::get_layers(*STDOUT, output=>1);
+  if ($unesc_unicode && $utf_output) {
+    # Probably it's safe to use wide characters
+    $COND_LB = "\N{LEFT DOUBLE PARENTHESIS}";
+    $COND_RB = "\N{RIGHT DOUBLE PARENTHESIS}";
+    $COND_MULT = "\N{MULTIPLICATION SIGN}";
+    $LQ = "«";
+    $RQ = "»";
+  } else {
+    $COND_LB = "(";
+    $COND_RB = ")";
+    $COND_MULT = "x";
+    $LQ = "<<";
+    $RQ = ">>";
+  }
+
   if ($debug) {
     our $_dbmaxlen = INT_MAX;
     btw "## DD result: fw1=",u($foldwidth1)," fw=",u($foldwidth)," pad='${pad}' maxll=$maxlinelen maxlNl=$maxlineNlen\n   result=",_dbrawstr($_);
@@ -1401,10 +1456,12 @@ sub _postprocess_DD_result {
 
     __unmagic_atom ;
     __unesc_unicode          if $unesc_unicode;
+    __unesc_nonoctal         if $octet_strings;
     __subst_controlpic_backesc      if $controlpics;
     __subst_spacedots        if $spacedots;
     __condense_strings(8)    if $condense_strings;
     __change_quotechars($qq) if $qq;
+    __nums_in_hex            if $nums_in_hex;
     __nums_with_underscores  if $underscores;
 
     if ($prepending) { $_ = $prepending . $_; $prepending = ""; }
@@ -1760,7 +1817,10 @@ sub _postprocess_DD_result {
   }
   elsif (index($listform,'l') >= 0) {
     # show as a bare list without brackets
-    $outstr =~ s/\A(?:${addrvis_re})?[\[\{]// && $outstr =~ s/[\]\}]\z//s or oops;
+    $outstr =~ s/\A(?:${addrvis_re})?\[(.*)\]\z/$1/s;
+    $outstr =~ s/\A(?:${addrvis_re})?\{(.*)\}\z/$1/s;
+    # or a single string without "quote marks"
+    $outstr =~ s/\A"(.*)"\z/$1/s;
   }
 
   # Insert user-specified padding after each embedded newline
@@ -1855,7 +1915,7 @@ sub _Interpolate {
       }
       else {
         if (/^.+?(?<!\\)([\$\@\%])/) {
-          confess __PACKAGE__." bug: Missed '$1' in «$_»"
+          confess __PACKAGE__." bug: Missed '$1' in ${LQ}$_${RQ}"
         }
         # Due to the need to simplify the big regexp above, \x{abcd} is now
         # split into "\x" and "{abcd}".  Combine consecutive pass-thrus
@@ -1876,7 +1936,7 @@ sub _Interpolate {
         $e = "Invalid expression syntax starting at '$leftover' in $funcname arg"
       } else {
         # Otherwise we may have a parser bug
-        $e = "Invalid expression (or ".__PACKAGE__." bug):\n«$leftover»";
+        $e = "Invalid expression (or ".__PACKAGE__." bug):\n${LQ}$leftover${RQ}";
       }
       carp "$e\n";
       push @pieces, ['p',"<INVALID EXPRESSION>".$leftover];
@@ -2199,47 +2259,43 @@ The available modifier characters are:
 
 =over 2
 
-B<l> - omit parenthesis to return a bare list (with "avis" or "hvis")
+B<l> - omit parenthesis to return a bare list with "avis" or "hvis"; omit quotes from a string formatted by "vis".
 
-B<o> - show object internals
+B<o> - show object internals (see C<Objects>);
 
-=over
 
-Calling B<< Objects(0) >> using the OO api has the same effect.
+B<r> - show abbreviated addresses of refs (see C<Refaddr>).
 
-=back
+B<< <NUMBER> >> - limit structure depth to <NUMBER> levels (see C<Maxdepth>).
+
+See C<Useqq> for more info about these:
+
+B<c> - Show control characters as "Control Picture" characters
+
+B<C> - condense strings of repeated characters
+
+B<h> - show numbers > 9 in hexadecimal
+
+B<O> - Optimize for strings containing binary octets.
 
 B<q> - show strings 'single quoted' if possible
 
 =over
 
-Internally, Data::Dumper is called with C<Useqq(0)>, but depending
+With B<q> Data::Dumper is called with C<Useqq(0)>, but depending
 on the version of Data::Dumper the result may be "double quoted"
 anyway if wide characters are present.
 
 =back
 
-B<r> - show abbreviated addresses of objects and other refs
-
-=over
-
-Calling B<< Refaddr(1) >> using the OO api has the same effect.
-
-=back
-
-B<< <NUMBER> >> - limit nested structure depth to <NUMBER> levels
-
-=over
-
-Calling B<< Maxdepth(NUMBER) >> using the OO api has the same effect.
-
-=back
+B<u> - show numbers with underscores between groups of three digits
 
 =back
 
 Functions must be imported explicitly
-unless they are imported by default (see list below)
-or created via the :all tag.
+unless they are imported by default (see list below).
+
+=for HIDE or created via the :all tag.
 
 To avoid having to import functions in advance, you can
 use them as methods and import only the C<visnew> function:
@@ -2264,31 +2320,32 @@ via the AUTOLOAD mechanism).
  addrvis addrvisl
  u quotekey qsh qshlist qshpath
 
-=head2 The :all import tag
-Z<> Z<>
-
-  use Data::Dumper::Interp qw/:all/;
-
-This generates and imports all possible variations using only suffix
-characters, in alphabetical order, with NUMBER <= 2.
-There are 119 variations, too many to remember.
-
-You only need to know the basic names
-
-  ivis, dvis, vis, avis, and hvis
-
-and the possible suffixes and their
-order (I<< <NUMBER> >>,C<l>,C<o>,C<q>,C<r>).
-
-For example, one function is C<< B<avis2lq> >>, which
-
- * Formats multiple arguments as an array ('avis')
- * Decends at most 2 levels into structures ('2')
- * Returns a comma-separated list *without* parenthesis ('l')
- * Shows strings in single-quoted form ('q')
-
-You could have used alternate names for the same function such as C<avis2ql>,
-C<q2avisl>, C<q_2_avis_l> etc. if called as methods or explicitly imported.
+=for HIDE =head2 The :all import tag
+=for HIDE Z<> Z<>
+=for HIDE
+=for HIDE   use Data::Dumper::Interp qw/:all/;
+=for HIDE
+=for HIDE This generates and imports methods uing all possible combinations of
+=for HIDE I<< <NUMBER> >>,C<l>,C<o>,C<q>, and C<r>,
+=for HIDE in alphabetical order, with NUMBER <= 2.
+=for HIDE There are 119 variations, too many to remember.
+=for HIDE
+=for HIDE You only need to know the basic names
+=for HIDE
+=for HIDE   ivis, dvis, vis, avis, and hvis
+=for HIDE
+=for HIDE and the possible suffixes and their
+=for HIDE order (I<< <NUMBER> >>,C<l>,C<o>,C<q>,C<r>).
+=for HIDE
+=for HIDE For example, one function is C<< B<avis2lq> >>, which
+=for HIDE
+=for HIDE  * Formats multiple arguments as an array ('avis')
+=for HIDE  * Decends at most 2 levels into structures ('2')
+=for HIDE  * Returns a comma-separated list *without* parenthesis ('l')
+=for HIDE  * Shows strings in single-quoted form ('q')
+=for HIDE
+=for HIDE You could have used alternate names for the same function such as C<avis2ql>,
+=for HIDE C<q2avisl>, C<q_2_avis_l> etc. if called as methods or explicitly imported.
 
 * To save memory, only stub declarations with prototypes are generated
 for imported functions.
@@ -2441,13 +2498,25 @@ the tiny "control picures" can be hard to read;
 to see traditional \n etc.  while still seeing wide characters as themselves,
 set C<Useqq> to just "unicode";
 
+=item "octets"
+
+Optimize for viewing binary strings (i.e. strings of octets, not "wide"
+characters).  Octal escapes are shown instead of \n, \r, etc.
+
 =item "spacedots"
 
 Space characters are shown as '·' (Middle Dot).
 
 =item "condense"
 
-Condense long strings.  For example, 42 repeated 'A's appear as "⸨Ax42⸩".
+Repeated sequences in strings are shown as "⸨I<char>xI<repcount>⸩".
+For example
+
+  vec(my $s, 31, 1) = 1;
+  say unpack "b*", ~$s;
+  say visnew->Useqq("unicode:condense")->visl(unpack "b*", ~$s);
+    -->11111111111111111111111111111110
+    -->⸨1×31⸩0
 
 =item "underscores"
 
